from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.service import Service
from webdriver_manager.firefox import GeckoDriverManager
from capmonster_python import HCaptchaTask
from time import sleep


class HCaptchaSelenium:
    def __init__(self, _client_key, _headless):
        self.options = Options()
        self.options.headless = _headless
        self.user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0"
        self.captcha = HCaptchaTask(_client_key)
        self.browser = webdriver.Firefox(service=Service(GeckoDriverManager().install()), options=self.options)
        self.website_url = "http://hcaptcha.jawne.info.pl/"
        self.expected = "Your request have submitted successfully"

    def _get_site_key(self):
        self.browser.get(self.website_url)
        return self.browser.find_element(By.CLASS_NAME, "h-captcha").get_attribute("data-sitekey")

    def _solve_hcaptcha(self):
        self.captcha.set_user_agent(self.user_agent)
        print("# User agent setted")
        task_id = self.captcha.create_task(website_url=self.website_url,
                                           website_key=self._get_site_key())
        print("# Task created successfully with the following id: {}".format(task_id))
        return self.captcha.join_task_result(task_id=task_id, maximum_time=180).get("gRecaptchaResponse")

    def submit_form(self):
        self.browser.execute_script("document.querySelector(\"textarea[name='g-recaptcha-response'"
                                    "]\").value = '{}'".format(self._solve_hcaptcha()))
        print("# Response received and placed to textarea")
        self.browser.find_element(By.ID, "fname").send_keys("xxx")
        self.browser.find_element(By.CSS_SELECTOR, "input[type='submit']").click()
        sleep(5)
        source = self.browser.page_source
        self.browser.close()
        return source


if __name__ == "__main__":
    from os import environ
    client_key = environ["API_KEY"]
    headless = environ["HEADLESS"]
    environ["WDM_LOG_LEVEL"] = "0"
    hcaptcha_selenium = HCaptchaSelenium(client_key, headless)
    assert hcaptcha_selenium.expected in hcaptcha_selenium.submit_form()
    print("# Submit succeed, test is OK")



from .recaptcha_v2 import RecaptchaV2Task
from .fun_captcha import FuncaptchaTask
from .recaptcha_v3 import RecaptchaV3Task
from .hcaptcha import HCaptchaTask
from .image_to_text import ImageToTextTask
from .geetest import GeeTestTask
from .utils import CapmonsterException



from functools import wraps


class CapmonsterException(Exception):
    def __init__(self, error_id, error_code, error_description, *args, **kwargs):
        super(CapmonsterException, self).__init__("[{}:{}]{}".format(error_code, error_id, error_description))
        self.error_description = error_description
        self.error_id = error_id
        self.error_code = error_code

    def __str__(self):
        return "[{}] {}".format(self.error_code, self.error_description)


def check_response():
    def checker(f):
        @wraps(f)
        def wrap(*args, **kwargs):
            rf = f(*args, **kwargs)
            if type(rf) == dict:
                if rf.get("errorId") == 0:
                    return rf
                else:
                    raise CapmonsterException(error_id=rf.get("errorId"),
                                              error_code=rf.get("errorCode"),
                                              error_description=rf.get("errorDescription"))
            else:
                raise CapmonsterException(error_id=-1,
                                          error_code="CAPMONSTER_API_ERROR",
                                          error_description="Sometimes can be happen if capmonster_python "
                                                            "servers there is too much intensity")
        return wrap
    return checker



import re
import requests
import unittest
from inspect import stack
from os import environ
from capmonster_python import *

client_key = environ.get("KEY")
proxy = environ.get("PROXY").split(",")
acceptable_error_codes = ["ERROR_CAPTCHA_UNSOLVABLE", "ERROR_MAXIMUM_TIME_EXCEED", "ERROR_NO_SLOT_AVAILABLE"]


class TestImageToText(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestImageToText, self).__init__(*args, **kwargs)
        self.captcha = ImageToTextTask(client_key)

    def test_text_captcha(self):
        task_id = self.captcha.create_task(image_path="tests/utils/text_captcha.png", recognizing_threshold=50)
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("text", solution)
        del solution, task_id

    def test_encoded_image_input(self):
        with open("tests/utils/text_captcha_encoded.txt", "r") as tce:
            captcha_img = tce.read()
        task_id = self.captcha.create_task(base64_encoded_image=captcha_img, recognizing_threshold=50)
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("text", solution)
        del solution, task_id, captcha_img

    def test_numeric_captcha(self):
        task_id = self.captcha.create_task(image_path="tests/utils/numeric_captcha.png", recognizing_threshold=50,
                                           numeric=1)
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("text", solution)
        del solution, task_id


class TestRecaptchaV2(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestRecaptchaV2, self).__init__(*args, **kwargs)
        self.captcha = RecaptchaV2Task(client_key)
        self.dump_cookies = {
            "cookie_1": "value_1",
            "cookie_2": "value_2",
            "cookie_3": "value_3"
        }

    def test_proxyless_recaptchav2(self):
        task_id = self.captcha.create_task("https://lessons.zennolab.com/captchas/recaptcha/v2_simple.php?level=high",
                                           "6Lcg7CMUAAAAANphynKgn9YAgA4tQ2KI_iqRyTwd",
                                           cookies=self.dump_cookies)
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("gRecaptchaResponse", solution)
        del task_id, solution

    def test_proxy_recaptchav2(self):
        try:
            self.captcha.set_user_agent("Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0")
            self.captcha.set_proxy(proxy[0], proxy[1], int(proxy[2]), proxy[3], proxy[4])
            task_id = self.captcha.create_task(
                "https://lessons.zennolab.com/captchas/recaptcha/v2_simple.php?level=high",
                "6Lcg7CMUAAAAANphynKgn9YAgA4tQ2KI_iqRyTwd",
                                               cookies=self.dump_cookies, no_cache=True)
            self.assertIs(type(task_id), int)
            solution = self.captcha.join_task_result(task_id)
            self.assertIs(type(solution), dict)
            self.assertIn("gRecaptchaResponse", solution)
            self.captcha.disable_proxy()
            self.captcha.reset_user_agent()
        except CapmonsterException as err:
            if any(err.error_code in s for s in acceptable_error_codes):
                print("Raised error code in " + stack()[0][3] + " but it's ok: {}".format(err.error_code))
                pass


class TestRecaptchaV3(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestRecaptchaV3, self).__init__(*args, **kwargs)
        self.captcha = RecaptchaV3Task(client_key)

    def test_recaptchav3(self):
        task_id = self.captcha.create_task("https://lessons.zennolab.com/captchas/recaptcha/v3.php?level=beta",
                                           "6Le0xVgUAAAAAIt20XEB4rVhYOODgTl00d8juDob", 0.6, "myverify")
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("gRecaptchaResponse", solution)
        del solution, task_id


class TestFuncaptchaTask(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestFuncaptchaTask, self).__init__(*args, **kwargs)
        self.captcha = FuncaptchaTask(client_key)
        self.dump_cookies = {
            "cookie_1": "value_1",
            "cookie_2": "value_2",
            "cookie_3": "value_3"
        }

    def test_proxyless_funcaptcha(self):
        task_id = self.captcha.create_task("https://funcaptcha.com/fc/api/nojs/?pkey=69A21A01-CC7B-B9C6-0F9A"
                                           "-E7FA06677FFC", "69A21A01-CC7B-B9C6-0F9A-E7FA06677FFC",
                                           data_blob="{\"blob\":\"dyXvXANMbHj1iDyz.Qj97JtSqR2n%2BuoY1V%2FbdgbrG7p"
                                                     "%2FmKiqdU9AwJ6MifEt0np4vfYn6TTJDJEfZDlcz9Q1XMn9przeOV%2FCr2"
                                                     "%2FIpi%2FC1s%3D\"}")
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("token", solution)
        del solution, task_id

    def test_proxy_funcaptcha(self):
        try:
            self.captcha.set_user_agent("Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) "
                                        "Gecko/20100101 Firefox/91.0")
            self.captcha.set_proxy(proxy[0], proxy[1], int(proxy[2]), proxy[3], proxy[4])
            task_id = self.captcha.create_task("https://funcaptcha.com/fc/api/nojs/?pkey=69A21A01-CC7B-B9C6-0F9A"
                                               "-E7FA06677FFC", "69A21A01-CC7B-B9C6-0F9A-E7FA06677FFC",
                                               data_blob="{\"blob\":\"dyXvXANMbHj1iDyz.Qj97JtSqR2n%2BuoY1V%2FbdgbrG7p"
                                                         "%2FmKiqdU9AwJ6MifEt0np4vfYn6TTJDJEfZDlcz9Q1XMn9przeOV%2FCr2"
                                                         "%2FIpi%2FC1s%3D\"}")
            self.assertIs(type(task_id), int)
            solution = self.captcha.join_task_result(task_id)
            self.assertIs(type(solution), dict)
            self.assertIn("token", solution)
            self.captcha.disable_proxy()
            self.captcha.reset_user_agent()
            del solution, task_id
        except CapmonsterException as err:
            if any(err.error_code in s for s in acceptable_error_codes):
                print("Raised error code in " + stack()[0][3] + " but it's ok: {}".format(err.error_code))
                pass


class TestHCaptcha(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestHCaptcha, self).__init__(*args, **kwargs)
        self.captcha = HCaptchaTask(client_key)
        self.dump_cookies = {
            "cookie_1": "value_1",
            "cookie_2": "value_2",
            "cookie_3": "value_3"
        }

    def test_proxyless_hcaptcha(self):
        self.captcha.set_user_agent("Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0")
        task_id = self.captcha.create_task("https://lessons.zennolab.com/captchas/hcaptcha/?level=easy",
                                           "472fc7af-86a4-4382-9a49-ca9090474471", cookies=self.dump_cookies)
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("gRecaptchaResponse", solution)
        del solution, task_id

    def test_proxy_hcaptcha(self):
        self.captcha.set_proxy(proxy[0], proxy[1], int(proxy[2]), proxy[3], proxy[4])
        task_id = self.captcha.create_task("https://lessons.zennolab.com/captchas/hcaptcha/?level=moderate",
                                           "d391ffb1-bc91-4ef8-a45a-2e2213af091b", cookies=self.dump_cookies)
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("gRecaptchaResponse", solution)
        self.captcha.disable_proxy()
        self.captcha.reset_user_agent()
        del solution, task_id

    def test_invisible_hcaptcha(self):
        task_id = self.captcha.create_task(
            "https://lessons.zennolab.com/captchas/hcaptcha/invisible.php?level=moderate",
            "d391ffb1-bc91-4ef8-a45a-2e2213af091b", is_invisible=True)
        self.assertIs(type(task_id), int)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("gRecaptchaResponse", solution)
        del solution, task_id


class TestGeeTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestGeeTest, self).__init__(*args, **kwargs)
        self.captcha = GeeTestTask(client_key)

    def getProperties(self):
        r = requests.get("https://2captcha.com/tr/demo/geetest").text
        return re.search("gt: '(.+?)'", r).group(1), re.search("challenge: '(.+?)'", r).group(1)

    def test_proxy_geetest(self):
        try:
            self.captcha.set_user_agent("Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0")
            self.captcha.set_proxy(proxy[0], proxy[1], int(proxy[2]), proxy[3], proxy[4])
            gt, challenge = self.getProperties()
            task_id = self.captcha.create_task(website_url="https://2captcha.com/tr/demo/geetest",
                                               gt=gt, challenge=challenge)
            solution = self.captcha.join_task_result(task_id)
            self.assertIs(type(solution), dict)
            self.assertIn("challenge", solution)
            self.assertIn("seccode", solution)
            self.assertIn("validate", solution)
            self.captcha.disable_proxy()
            self.captcha.reset_user_agent()
            del solution, task_id
        except CapmonsterException as err:
            if any(err.error_code in s for s in acceptable_error_codes):
                print("Raised error code in " + stack()[0][3] + " but it's ok: {}".format(err.error_code))
                pass

    def test_proxyless_geetest(self):
        try:
            gt, challenge = self.getProperties()
            task_id = self.captcha.create_task(website_url="https://2captcha.com/tr/demo/geetest",
                                               gt=gt, challenge=challenge)
            solution = self.captcha.join_task_result(task_id)
            self.assertIs(type(solution), dict)
            self.assertIn("challenge", solution)
            self.assertIn("seccode", solution)
            self.assertIn("validate", solution)
            del solution, task_id
        except CapmonsterException as err:
            if any(err.error_code in s for s in acceptable_error_codes):
                print("Raised error code in " + stack()[0][3] + " but it's ok: {}".format(err.error_code))
                pass


if __name__ == "__main__":
    unittest.main()



from .capmonster import Capmonster


class RecaptchaV3Task(Capmonster):
    def __init__(self, client_key):
        super().__init__(client_key)

    def create_task(self, website_url: str, website_key: str, minimum_score: float = 0.3,
                    page_action: str = None, no_cache: bool = None):
        data = {
            "clientKey": self._client_key,
            "task": {
                "type": "RecaptchaV3TaskProxyless",
                "websiteURL": website_url,
                "websiteKey": website_key
            }
        }
        if 0.1 <= minimum_score <= 0.9:
            data["task"]["minScore"] = minimum_score
        if page_action is not None:
            data["task"]["pageAction"] = page_action
        if no_cache:
            data["task"]["nocache"] = no_cache
        return self._make_request("createTask", data).get("taskId")



import unittest
from os import environ
from capmonster_python import ImageToTextTask

client_key = environ["KEY"]


class TestMethods(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestMethods, self).__init__(*args, **kwargs)
        self.captcha = ImageToTextTask(client_key)

    def test_get_balance(self):
        balance = self.captcha.get_balance()
        self.assertIs(type(balance), float)

    def test_join_task_result(self):
        task_id = self.captcha.create_task(image_path="tests/utils/text_captcha.png", recognizing_threshold=50)
        solution = self.captcha.join_task_result(task_id)
        self.assertIs(type(solution), dict)
        self.assertIn("text", solution)


if __name__ == "__main__":
    unittest.main()



from .capmonster import Proxy, UserAgent
from .utils import CapmonsterException
from typing import Union


class HCaptchaTask(UserAgent, Proxy):
    def __init__(self, client_key):
        super(HCaptchaTask, self).__init__(client_key)

    def create_task(self, website_url: str, website_key: str, is_invisible: bool = None, custom_data: str = None,
                    cookies: Union[dict, list, str] = None, no_cache: bool = None):
        data = {
            "clientKey": self._client_key,
            "task": {
                "type": "HCaptchaTask",
                "websiteURL": website_url,
                "websiteKey": website_key
            }
        }
        if cookies is not None:
            data = self._add_cookies(cookies, data)
        if is_invisible is not None:
            data["task"]["isInvisible"] = is_invisible
        if custom_data is not None:
            data, is_user_agent = self._add_user_agent(data)
            if is_user_agent is False:
                raise CapmonsterException(-1,
                                          "USER_AGENT_ERROR",
                                          "You must provide an user agent if you submit captcha with custom_data")
            data["task"]["data"] = custom_data
        if custom_data is None:
            data, is_user_agent = self._add_user_agent(data)
        data, is_proxy = self._is_proxy_task(data)
        if no_cache:
            data["task"]["nocache"] = no_cache
        return self._make_request("createTask", data).get("taskId")



from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.firefox.service import Service
from webdriver_manager.firefox import GeckoDriverManager
from capmonster_python import RecaptchaV2Task
from time import sleep


class RecaptchaV2Selenium:
    def __init__(self, _client_key, _headless):
        self.options = Options()
        self.options.headless = _headless
        self.user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0"
        self.captcha = RecaptchaV2Task(_client_key)
        self.browser = webdriver.Firefox(service=Service(GeckoDriverManager().install()), options=self.options)
        self.website_url = "https://www.google.com/recaptcha/api2/demo"

    def _get_site_key(self):
        self.browser.get("https://www.google.com/recaptcha/api2/demo")
        return self.browser.find_element(By.ID, "recaptcha-demo").get_attribute("data-sitekey")

    def _solve_recaptcha(self):
        self.captcha.set_user_agent(self.user_agent)
        task_id = self.captcha.create_task(website_url=self.website_url,
                                           website_key=self._get_site_key(),
                                           no_cache=True)
        print("# Task created successfully with the following id: {}".format(task_id))
        return self.captcha.join_task_result(task_id=task_id, maximum_time=180).get("gRecaptchaResponse")

    def submit_form(self):
        self.browser.execute_script("document.getElementsByClassName('g-recaptcha-response')[0].innerHTML = "
                                    f"'{self._solve_recaptcha()}';")
        print("# Response received and placed to g-recaptcha-response textarea")
        self.browser.find_element(By.ID, "recaptcha-demo-submit").click()
        sleep(5)
        source = self.browser.find_element(By.CLASS_NAME, "recaptcha-success")
        self.browser.close()
        return source


if __name__ == "__main__":
    from os import environ
    client_key = environ["API_KEY"]
    headless = environ["HEADLESS"]
    environ["WDM_LOG_LEVEL"] = "0"
    recaptcha_selenium = RecaptchaV2Selenium(client_key, headless)
    assert recaptcha_selenium.submit_form() is not None
    print("# Submit is succeed, test is OK")



from capmonster_python import ImageToTextTask


class TextNumericExample:
    def __init__(self, _client_key: str):
        self.captcha = ImageToTextTask(_client_key)
        self.image_path_text = "examples/utils/text_captcha.png"
        self.image_path_numeric = "examples/utils/numeric_captcha.png"
        self.expected_1 = "fdamc"
        self.expected_2 = "246477"

    def solve_text(self):
        task_id = self.captcha.create_task(image_path=self.image_path_text, recognizing_threshold=70)
        print("# Task created successfully with the following id: {}".format(task_id))
        result = self.captcha.join_task_result(task_id=task_id)
        print("# Response received: {}".format(result.get("text")))
        return result.get("text")

    def solve_numeric(self):
        task_id = self.captcha.create_task(image_path=self.image_path_numeric, recognizing_threshold=70, numeric=1)
        print("# Task created successfully with the following id: {}".format(task_id))
        result = self.captcha.join_task_result(task_id=task_id)
        print("# Response received: {}".format(result.get("text")))
        return result.get("text")


if __name__ == "__main__":
    from os import environ
    client_key = environ["API_KEY"]
    example_text_numeric = TextNumericExample(client_key)
    assert example_text_numeric.expected_1 in example_text_numeric.solve_text()
    print("# Alphanumeric test is OK")
    assert example_text_numeric.expected_2 in example_text_numeric.solve_numeric()
    print("# Numeric test is OK")



from .capmonster import Proxy, UserAgent
from typing import Union


class FuncaptchaTask(UserAgent, Proxy):
    def __init__(self, client_key):
        super(FuncaptchaTask, self).__init__(client_key)

    def create_task(self, website_url: str, website_public_key: str, api_js_subdomain: str = None,
                    data_blob: str = None, cookies: Union[dict, list, str] = None, no_cache: bool = None):
        data = {
            "clientKey": self._client_key,
            "task": {
                "type": "FunCaptchaTask",
                "websiteURL": website_url,
                "websitePublicKey": website_public_key
            }
        }
        if data_blob is not None:
            data["task"]["data"] = data_blob
        data, is_proxy = self._is_proxy_task(data)
        if is_proxy:
            data, is_user_agent = self._add_user_agent(data)
            if cookies is not None:
                data = self._add_cookies(cookies, data)
        if api_js_subdomain is not None:
            data["task"]["funcaptchaApiJSSubdomain"] = api_js_subdomain
        if no_cache:
            data["task"]["nocache"] = no_cache
        return self._make_request("createTask", data).get("taskId")



from .capmonster import Capmonster, CapmonsterException
from base64 import b64encode


class ImageToTextTask(Capmonster):
    def __init__(self, client_key):
        super().__init__(client_key)

    def create_task(self, image_path: str = None, base64_encoded_image: str = None, module: str = None,
                    recognizing_threshold: int = None, case: bool = None, numeric: int = None,
                    math: bool = None):
        if base64_encoded_image is None and image_path is None:
            raise CapmonsterException(error_id=-1,
                                      error_code="ERROR_NOTHING_GIVEN",
                                      error_description="You have to give image_path or base64_encoded_image")
        data = {
            "clientKey": self._client_key,
            "task": {
                "type": "ImageToTextTask"
            }
        }
        if base64_encoded_image is None:
            img_body = self._from_path(image_path)
            data["task"]["body"] = img_body
        else:
            data["task"]["body"] = base64_encoded_image
        if module is not None:
            data["task"]["CapMonsterModule"] = module
        if recognizing_threshold is not None and (0 <= recognizing_threshold <= 100):
            data["task"]["recognizingThreshold"] = recognizing_threshold
        if case is not None:
            data["task"]["Case"] = case
        if numeric is not None and (0 <= numeric <= 1):
            data["task"]["numeric"] = numeric
        if math is not None:
            data["task"]["math"] = math
        return self._make_request("createTask", data).get("taskId")

    @staticmethod
    def _from_path(image_path: str):
        with open(image_path, "rb") as img:
            base64_img = b64encode(img.read()).decode("ascii")
        return base64_img



import requests
import re
from capmonster_python import FuncaptchaTask


class FuncaptchaRequest:
    def __init__(self, _client_key: str):
        self.captcha = FuncaptchaTask(_client_key)
        self.s = requests.Session()
        self.user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0"
        self.s.headers = {"User-Agent": self.user_agent}
        self.website_url = "https://client-demo.arkoselabs.com/solo-animals"
        self.expected = "Solved!"

    def _form_html(self):
        return self.s.get(self.website_url).text

    def _site_token(self):
        site_key = re.search("public_key: \"(.+?)\"", self._form_html()).group(1)
        print("# Site key is found: {}".format(site_key))
        task_id = self.captcha.create_task(website_url=self.website_url, website_public_key=site_key)
        print("# Task created successfully with the following id: {}".format(task_id))
        result = self.captcha.join_task_result(task_id=task_id)
        print("# Response received")
        return result.get("token")

    def submit_form(self):
        result = self._site_token()
        data = {
            "name": "xx",
            "fc-token": result
        }
        return self.s.post(f"{self.website_url}/verify", data=data).text


if __name__ == "__main__":
    from os import environ
    client_key = environ["API_KEY"]
    example_request = FuncaptchaRequest(client_key)
    assert example_request.expected in example_request.submit_form()
    print("# Submit succeed, test is OK")



import requests
from time import sleep
from .utils import *


class Capmonster:
    __SOFT_ID = 30
    _HOST_URL = "https://api.capmonster.cloud"
    _CREATE_TASK_URL = "/createTask"
    _TASK_RESULT_URL = "/getTaskResult"
    _BALANCE_URL = "/getBalance"

    def __init__(self, client_key):
        self._client_key = client_key

    def get_balance(self):
        data = {"clientKey": self._client_key}
        return self._make_request("getBalance", data).get("balance")

    def get_task_result(self, task_id: int):
        data = {
            "clientKey": self._client_key,
            "taskId": task_id
        }
        result = self._make_request("getTaskResult", data)
        is_ready = self._is_ready(result)
        if is_ready:
            return result.get("solution")
        else:
            return False

    def join_task_result(self, task_id: int, maximum_time: int = 120):
        for i in range(0, maximum_time + 1, 2):
            result = self.get_task_result(task_id)
            if result is not False and result is not None:
                return result
            elif result is False:
                i += 1
                sleep(2)
        raise CapmonsterException(61, "ERROR_MAXIMUM_TIME_EXCEED", "Maximum time is exceed.")

    @staticmethod
    def _is_ready(response: dict):
        status = response.get("status")
        if status == "processing":
            return False
        elif status == "ready":
            return True

    @check_response()
    def _make_request(self, method: str, data: dict):
        _method = None
        if method == "getBalance":
            _method = self._BALANCE_URL
        elif method == "getTaskResult":
            _method = self._TASK_RESULT_URL
        elif method == "createTask":
            _method = self._CREATE_TASK_URL
            data["softId"] = self.__SOFT_ID
        try:
            response = requests.post("{}{}".format(self._HOST_URL, _method), json=data).json()
        except Exception as err:
            raise CapmonsterException(-1, type(err).__name__, str(err))
        return response

    @staticmethod
    def _add_cookies(cookies, data):
        if cookies is None:
            return data
        str_cookies = ""
        if type(cookies) == dict:
            for key, value in cookies.items():
                if value == list(cookies.items())[-1][1]:
                    str_cookies += "{}={}".format(key, value)
                else:
                    str_cookies += "{}={};".format(key, value)
        elif type(cookies) == list:
            for i in cookies:
                if not len(cookies) % 2 == 0:
                    raise AttributeError("List cookies length must be even numbers")
                if cookies.index(i) % 2 == 0:
                    str_cookies += "{}=".format(i)
                elif cookies[cookies.index(i)] == cookies[-1]:
                    str_cookies += "{}".format(i)
                elif cookies.index(i) % 2 == 1:
                    str_cookies += "{};".format(i)
        elif type(cookies) == str:
            data["task"]["cookies"] = cookies
            return data
        data["task"]["cookies"] = str_cookies
        return data


class Proxy(Capmonster):
    def __init__(self, client_key):
        super().__init__(client_key)
        self._proxy_type = None
        self._proxy_address = None
        self._proxy_port = None
        self._proxy_login = None
        self._proxy_password = None

    def set_proxy(self, proxy_type: str, proxy_address: str, proxy_port: int,
                  proxy_login: str = None, proxy_password: str = None):
        self._proxy_type = proxy_type
        self._proxy_address = proxy_address
        self._proxy_port = proxy_port
        self._proxy_login = proxy_login
        self._proxy_password = proxy_password

    def disable_proxy(self):
        self._proxy_type = None
        self._proxy_address = None
        self._proxy_port = None
        self._proxy_login = None
        self._proxy_password = None

    def _is_proxy_task(self, data: dict):
        """
        Determine for is this a proxy task or not?
        e.g if you are not set the values with set_proxy method, it is a proxyless method, or if you are set up it is a
        proxy task.
        """
        if self._proxy_type and self._proxy_address and self._proxy_port:
            data["task"]["proxyType"] = self._proxy_type
            data["task"]["proxyAddress"] = self._proxy_address
            data["task"]["proxyPort"] = self._proxy_port
            if self._proxy_login and self._proxy_password:
                data["task"]["proxyLogin"] = self._proxy_login
                data["task"]["proxyPassword"] = self._proxy_password
            return data, True
        data["task"]["type"] += "Proxyless"
        return data, False


class UserAgent(Capmonster):
    def __init__(self, client_key):
        super().__init__(client_key)
        self._user_agent = None

    def set_user_agent(self, user_agent: str):
        self._user_agent = user_agent

    def reset_user_agent(self):
        self._user_agent = None

    def _add_user_agent(self, data):
        if self._user_agent:
            data["task"]["userAgent"] = self._user_agent
            return data, True
        return data, False



import re
from selenium import webdriver
from selenium.webdriver.firefox.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.firefox.service import Service
from webdriver_manager.firefox import GeckoDriverManager
from capmonster_python import FuncaptchaTask
from time import sleep


class FuncaptchaSelenium:
    def __init__(self, _client_key, _headless):
        self.options = Options()
        self.options.headless = _headless
        self.user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0"
        self.captcha = FuncaptchaTask(_client_key)
        self.browser = webdriver.Firefox(service=Service(GeckoDriverManager().install()), options=self.options)
        self.website_url = "https://client-demo.arkoselabs.com/solo-animals"
        self.expected = "Solved!"

    def _get_site_key(self):
        self.browser.get(self.website_url)
        return re.search("public_key: \"(.+?)\",", self.browser.page_source).group(1)

    def _solve_funcaptcha(self):
        self.captcha.set_user_agent(self.user_agent)
        task_id = self.captcha.create_task(website_url=self.website_url,
                                           website_public_key=self._get_site_key())
        print("# Task created successfully with the following id: {}".format(task_id))
        return self.captcha.join_task_result(task_id=task_id, maximum_time=180).get("token")

    def submit_form(self):
        self.browser.execute_script("""
        document.getElementById('FunCaptcha-Token').value = decodeURIComponent('{0}')
        document.getElementById('verification-token').value = decodeURIComponent('{0}')
        document.getElementById('submit-btn').disabled = false
        """.format(self._solve_funcaptcha()))
        print("# Response received and placed to (FunCaptcha-Token and verifaction-token) textarea")
        self.browser.find_element(By.ID, "submit-btn").click()
        sleep(5)
        h3_text = self.browser.find_element(By.CSS_SELECTOR, "h3").text
        self.browser.close()
        return h3_text


if __name__ == "__main__":
    from os import environ
    client_key = environ["API_KEY"]
    headless = environ["HEADLESS"]
    environ["WDM_LOG_LEVEL"] = "0"
    funcaptcha_selenium = FuncaptchaSelenium(client_key, headless)
    assert funcaptcha_selenium.expected in funcaptcha_selenium.submit_form()
    print("# Submit is succeed, test is OK")



from .capmonster import Proxy, UserAgent
from typing import Union


class RecaptchaV2Task(UserAgent, Proxy):
    def __init__(self, client_key):
        super(RecaptchaV2Task, self).__init__(client_key)

    def create_task(self, website_url: str, website_key: str,
                    cookies: Union[dict, list, str] = None, recaptcha_s_value: str = None,
                    no_cache: bool = None):
        data = {
            "clientKey": self._client_key,
            "task": {
                "type": "NoCaptchaTask",
                "websiteURL": website_url,
                "websiteKey": website_key
            }
        }
        data, is_proxy = self._is_proxy_task(data)
        data, is_user_agent = self._add_user_agent(data)
        data = self._add_cookies(cookies, data)
        if recaptcha_s_value is not None:
            data["task"]["recaptchaDataSValue"] = recaptcha_s_value
        if no_cache:
            data["task"]["nocache"] = no_cache
        return self._make_request("createTask", data).get("taskId")



from distutils.core import setup
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()


setup(
    name="capmonster_python",
    packages=["capmonster_python"],
    url="https://github.com/alperensert/capmonster_python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    author="Alperen Sert",
    author_email="mail@alperenn.com",
    description="capmonster.cloud library for Python",
    requires=["requests"],
    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    use_scm_version=True,
    test_suite="tests",
    setup_requires=["setuptools_scm", "wheel"],
    install_requires=["requests", "six"],
    python_requires=">=3",
    project_urls={
        "Documentation": 'https://github.com/alperensert/capmonster_python/wiki/',
        "Source": 'https://github.com/alperensert/capmonster_python/',
        "Tracker": 'https://github.com/alperensert/capmonster_python/issues',
    },
)


from .capmonster import Proxy, UserAgent


class GeeTestTask(UserAgent, Proxy):
    def __init__(self, client_key):
        super(GeeTestTask, self).__init__(client_key)

    def create_task(self, website_url: str, gt: str, challenge: str,
                    api_server_subdomain: str = None, get_lib: str = None):
        data = {
            "clientKey": self._client_key,
            "task": {
                "type": "GeeTestTask",
                "websiteURL": website_url,
                "gt": gt,
                "challenge": challenge
            }
        }
        data, is_user_agent = self._add_user_agent(data)
        data, is_proxy = self._is_proxy_task(data)
        if api_server_subdomain:
            data["task"]["geetestApiServerSubdomain"] = api_server_subdomain
        if get_lib:
            data["task"]["geetestGetLib"] = get_lib
        return self._make_request("createTask", data).get("taskId")

